#include "StdAfx.h"
#include ".\expconditionset.h"

IMPLEMENT_SERIAL( ExpConditionSet, CObject, 1 )

ExpConditionSet::ExpConditionSet( int foo )
: m_size(0)
{
}

ExpConditionSet::~ExpConditionSet(void)
{
	for ( int i=0; i < returnSize( ); i++ )
	{
		delete at(i);
	}
}

void ExpConditionSet::clear( )
{
	for( int i=0; i < conditions.GetSize( ); i++ )
	{
		delete conditions[i];
	}
	
	conditions.RemoveAll( );
	m_size = 0;
}

void ExpConditionSet::addCondition(char *name)
{
	conditions.Add( new ExpCondition( name ) );
	m_size++;
}

void ExpConditionSet::addCondition( ExpCondition *condition )
{
	conditions.Add( new ExpCondition( condition ) );
	m_size++;
}

void ExpConditionSet::insertCondition( int where, ExpCondition *condition )
{
	conditions.InsertAt( where, new ExpCondition( condition ) );
	m_size++;
}

void ExpConditionSet::deleteLastCondition()
{
	delete at( returnSize( ) - 1 );
	conditions.RemoveAt( returnSize( ) - 1 );
	m_size--;
}

void ExpConditionSet::deleteCondition( int where )
{
	delete at( where );
	conditions.RemoveAt( where );
	m_size--;
}

void ExpConditionSet::copyLastCondition()
{
	conditions.Add( new ExpCondition( at( returnSize( ) - 1 ) ) );
	m_size++;
}

void ExpConditionSet::copyLastConditionMinimal()
{
	conditions.Add( new ExpCondition( at( returnSize( ) - 1 ), 1 ) );
	m_size++;
}

void ExpConditionSet::copyConditionMinimal( int where )
{
	conditions.InsertAt( where + 1, new ExpCondition( at( where ), 1 ) );
	m_size++;
}

void ExpConditionSet::copyCondition( int condNumber)
{
	conditions.Add( new ExpCondition( at( condNumber ) ) );
	m_size++;
}

void ExpConditionSet::copyCondition( int where, int condNumber )
{
	conditions.InsertAt( where, new ExpCondition( at( condNumber ) ) );
	m_size++;
}

int ExpConditionSet::checkAllConditions( WTextFile *theTextFile )
{
	int errorCode;
	int maxNumberOfNeededLines = 1;
	
	for (int i=0; i < returnSize( ); i++)
	{
		at(i)->checkArray(theTextFile);

		int numberOfNeededLines;

		if( at( i )->m_centTendency == 6 ) // raw values
			numberOfNeededLines = at( i )->m_relevantAVs.GetSize( );
		else
			numberOfNeededLines = 1;

		if( numberOfNeededLines > maxNumberOfNeededLines )
			maxNumberOfNeededLines = numberOfNeededLines;

		at(i)->computeMean( &errorCode );
	}

	return maxNumberOfNeededLines;
}

int ExpConditionSet::WriteAllConditionsToOutFile(OutFile *outFile, int lineNumber, int columnShift)
{
	int maxNumberOfNeededLines = 1;
	
	int i;
	for (i=0; i < returnSize(); i++)
	{
		int numberOfNeededLines = at(i)->WriteConditionToOutFile( outFile, lineNumber, i + columnShift);

		if( numberOfNeededLines > maxNumberOfNeededLines )
			maxNumberOfNeededLines = numberOfNeededLines;
	}

	for( int j = 0; j < maxNumberOfNeededLines; j++ )
	{
		outFile->getWord("|||", lineNumber + j, i + columnShift );
	}

	// if lineNumber == 1, also finish head line
	if (lineNumber == 1)
		outFile->getWord("|||", 0, i+ columnShift );

	return maxNumberOfNeededLines;
}

int ExpConditionSet::returnSize( )
{
	return m_size;
}

ExpCondition *ExpConditionSet::at( int index )
{
	return conditions.GetAt( index );
}

void ExpConditionSet::Serialize( CArchive& archive )
{
	CObject::Serialize( archive );
	
	if( ! archive.IsStoring( ) ) // load
	{
		archive >> m_size;
		
		conditions.SetSize( m_size );
		for( int i=0; i < m_size; i++ )
		{
			conditions[i] = new ExpCondition;
		}
	}
	else // save
	{
		archive << m_size;
	}

	// load and save
	for( int i=0; i < m_size; i++ )
	{
		conditions[i]->Serialize( archive );
	}
}

InputError ExpConditionSet::checkValidInputs( WTextFile * theTextFile )
{
	InputError inputError;

	for( int i=0; i < returnSize(); i++ )
	{
		inputError = at( i )->checkValidInputs( theTextFile ); // ExpConditionSet::at( i ) = ExpCondition
		if( inputError.errorCode )
		{
			inputError.expCondition = i;
			break;
		}
	}

	return inputError;
}