// AuswertungDlg.h : Headerdatei
//

#pragma once


#include "GridCtrl_src\GridCtrl.h"
#include "AVGridCtrl.h"
#include "PrevGridCtrl.h"
#include "NewCellTypes\GridCellCombo.h"
#include "NewCellTypes\GridCellCheck.h"


#include <iostream>
#include <fstream>
#include "TextFile.h"
#include "WTextFile.h"
#include "ExpCondition.h"
#include "ExpConditionSet.h"
#include "UvSet.h"
#include "OutFile.h"
#include "MultiFileDialog.h"
#include "ComputedAv.h"
#include "SpecialAvDlg.h"


#include <string>
#include <vector>
#include "afxcmn.h"
#include "afxwin.h"

#include "OrSubCondition.h"
#include "InputError.h"

#include "ProtocolFileChange.h"

using namespace std;



class ScaleHelper
{
public:
	ScaleHelper()
	{
		m_nScaleFactor = 0;
	}

	UINT GetScaleFactor()
	{
		return m_nScaleFactor;
	}

	void SetScaleFactor(__in UINT iDPI)
	{
		m_nScaleFactor = MulDiv(iDPI, 100, 96);
	}

	int ScaleValue(int value)
	{
		return MulDiv(value, m_nScaleFactor, 100);
	}

	// Scale rectangle from raw pixels to relative pixels.
	void ScaleRectangle(__inout RECT *pRectangle)
	{
		pRectangle->left = ScaleValue(pRectangle->left);
		pRectangle->right = ScaleValue(pRectangle->right);
		pRectangle->top = ScaleValue(pRectangle->top);
		pRectangle->bottom = ScaleValue(pRectangle->bottom);
	}

	// Scale Point from raw pixels to relative pixels.
	void ScalePoint(__inout POINT *pPoint)
	{
		pPoint->x = ScaleValue(pPoint->x);
		pPoint->y = ScaleValue(pPoint->y);
	}

	HFONT CreateScaledFont(int nFontHeight)
	{
		int nScaledFontHeight = ScaleValue(nFontHeight);
		LOGFONT lf;
		memset(&lf, 0, sizeof(lf));
		lf.lfQuality = CLEARTYPE_QUALITY;
		lf.lfHeight = -nScaledFontHeight;
		auto hFont = CreateFontIndirect(&lf);
		return hFont;
	}

private:
	UINT m_nScaleFactor;
};


#define BASELINE_PADDING	15
#define BASELINE_WINDOW_WIDTH       800
#define BASELINE_WINDOW_HEIGHT      500
#define BASELINE_BUTTON_WIDTH       280
#define BASELINE_BUTTON_HEIGHT      60
#define BASELINE_FONT_HEIGHT 30





class CArtDlg : public CDialog
{
public:
	CArtDlg(CWnd* pParent = NULL);	// standard constructor

	enum { IDD = IDD_AUSWERTUNG_DIALOG };

	protected:
	virtual void DoDataExchange(CDataExchange* pDX);

	AVGridCtrl m_gridAv;
	CGridCtrl m_gridUv;
	PrevGridCtrl m_gridPreview;
	CGridCtrl m_gridConds;

protected:
	HICON m_hIcon;

	//virtual LRESULT WindowProc(UINT message,WPARAM wParam,LPARAM lParam);


	BOOL getdpi();
	virtual BOOL OnInitDialog();
	afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnDestroy();
	DECLARE_MESSAGE_MAP()
public:
	BOOL findDecimalSeparator ();
	afx_msg void OnBnClickedButtonStart();
	void pumpMessages();
	void setExpCondition(int conditionNumber);

	ExpConditionSet *m_condset;
	ExpConditionSet *m_copyCondset;
	UvSet *m_uvSet;
	void setSubCondition(int conditionNumber, int subCondNumber);
	CSpinButtonCtrl m_spinConditionNumber;
	void returnSubCondition(int conditionNumber, int subCondNumber, int startLine);
	void returnExpCondition( int conditionNumber );
	void setRadiosCentTendency( int centTendency );
	int m_numberOfConditions;
	int m_numberOfUvs;
	int m_numberOfRows;
	int m_numberOfCols;
	void initGridAv(void);
	void initGridUv(void);
	void initGridConds(void);
	void initControls(void);
	~CArtDlg(void);
	int m_actualConditionNumber;
	afx_msg void OnDeltaposSpinCondNumber(NMHDR *pNMHDR, LRESULT *pResult);
	CEdit m_editCondName;

	CArray<CString> m_filenames;
	CString m_outfile;
	CString m_artFilePath;
	BOOL m_somethingToSave;

	BOOL checkInputFiles(void);
	BOOL checkOutputFiles(void);
	void setUvs(void);
	void getUvs(void);
	int convertComparator(CString comparatorString);
	int convertReferenceType( CString refTypeString );
	CString convertComparator(int comparator);
	CString convertReferenceType( int referenceType );
	void OnGridEndSelChange(NMHDR *pNotifyStruct, LRESULT* /*pResult*/);
	void previewFile( int filenumber = 0 );

	CButton m_checkTabs;
	CButton m_checkBlanks;
	CButton m_checkCR;
	CButton m_checkNewLine;
	CButton m_checkVertTab;
	CButton m_checkNewPage;

	CStatic m_staticDelimiters;
	CStatic m_staticVertical;
	CStatic m_staticHorizontal;


	BOOL m_tabs, m_blanks, m_crs, m_newLines, m_vertTabs, m_newPages;

	afx_msg void OnBnClickedCheckTabs();
	afx_msg void OnBnClickedCheckBlanks();
	afx_msg void OnBnClickedButtonAvFromPreview();
	afx_msg void OnBnClickedButtonUvFromPreview();
	CEdit m_editColumnAv;
	void refreshCondsGrid( BOOL selectCell = FALSE );
	void OnCondsGridClick( NMHDR *pNotifyStruct, LRESULT* /*pResult*/ );
	afx_msg void OnFileOpen();
	void openArtFile( CString filePath );

	HACCEL m_hAccel;
	virtual BOOL PreTranslateMessage( MSG* pMsg );

	afx_msg void OnFileSave();
	afx_msg void OnEditCopySelection();
	afx_msg void OnEditCutSelection();
	afx_msg void OnEditPasteSelection();
	afx_msg void OnEditAppendSelection();
	afx_msg void OnWithinSubjectsInsertRule();
	afx_msg void OnWithinSubjectsInsertOrBlock();
	afx_msg void OnWithinSubjectsDeleteRule();
	afx_msg void OnWithinSubjectsAddCondition();
	afx_msg void OnWithinSubjectsCopyCondition();
	afx_msg void OnWithinSubjectsDeleteCondition();
	afx_msg void OnWithinSubjectsInsertLine();
	void OnWithinSubjectsDeleteLine( int row );
	afx_msg void OnWithinSubjectsDeleteLines();
	afx_msg void OnFileSelectInputFiles();
	afx_msg void OnFileSelectOutputFile();
	afx_msg void OnFileQuit();
	afx_msg void OnBetweenSubjectsAddLine();
	afx_msg void OnBetweenSubjectsDeleteLine( int theRow );
	void OnBetweenSubjectsDeleteLines();

	UINT m_width, m_height;
	UINT m_dpiX, m_dpiY;

	CEdit m_editLineFrom;
	CEdit m_editLineTo;

	CButton m_radioRawValues;
	CButton m_radioMean;
	CButton m_radioMedian;
	CButton m_radioIncidences;
	CButton m_radioSumme;
	CButton m_radioVariance;
	CButton m_radioStandardDeviation;
	CButton m_checkPopEst;

	afx_msg void OnBnClickedRadioMean();
	afx_msg void OnBnClickedRadioMedian();
	afx_msg void OnBnClickedOptions();
	void updateAvFieldsEnDisabled( int centTendency, BOOL specialAv, BOOL fileEnd );

	afx_msg void OnSize(UINT nType, int cx, int cy);
	void positionDialogControls();
	CButton m_buttonFromPreview;
	CStatic m_staticBetweenSubjects;
	CButton m_buttonStart;
	afx_msg void OnGetMinMaxInfo(MINMAXINFO* lpMMI);
	
public:
	afx_msg void OnBnClickedRadioIncidences();
public:
	SpecialAvDlg *m_specialAvDlg;
public:
	afx_msg void OnBnClickedCheckSpecialAv();
public:
	CButton m_buttonOptions;
public:
	CButton m_checkSpecialAv;
public:
	afx_msg void OnEditRedoLastStep();
	CStatic m_staticWithinSubjects;

	CButton m_buttonWithinFromPreview;
	CStatic m_staticLineFrom;
	CStatic m_staticLineTo;
	CStatic m_staticColumnAv;
	CStatic m_staticName;
	CStatic m_staticConditionNumber;
	afx_msg void OnStnClickedStaticName();

protected:
	CDC *m_pMemDC; // memory dc
	CBitmap *m_pBitmap, *m_pOldBitmap; // bitmap

public:
	CEdit m_editConditionNumber;
	afx_msg void OnFileSaveas();
	afx_msg void OnBnClickedCheckCr();
	afx_msg void OnBnClickedCheckNewLine();
	afx_msg void OnBnClickedCheckVertTab();
	afx_msg void OnBnClickedCheckNewPage();

protected:
	CEdit* GetEditFocused(void);
	BOOL IsEditSelection(void);
	BOOL EditHasText(void);

public:
	afx_msg void OnEditCut();
	afx_msg void OnEditCopy();
	afx_msg void OnEditPaste();

	afx_msg void OnUpdateEditPaste(CCmdUI *pCmdUI);
	afx_msg void OnInitMenuPopup(CMenu* pPopupMenu, UINT nIndex, BOOL bSysMenu);
	afx_msg void OnEditClear();
	afx_msg void OnEditClearall();
	afx_msg void OnEditSelectall();
	afx_msg void OnUpdateEditClearall(CCmdUI *pCmdUI);
	afx_msg void OnUpdateEditIsSel(CCmdUI *pCmdUI);
	afx_msg void OnUpdateEditSelectall(CCmdUI *pCmdUI);
	afx_msg void OnUpdateCondsToPaste(CCmdUI *pCmdUI);
	afx_msg void OnUpdateCondsSelected(CCmdUI *pCmdUI);
	afx_msg void OnUpdateAvLinesSelected(CCmdUI *pCmdUI);
	afx_msg void OnUpdateCondsAvailable(CCmdUI *pCmdUI);
	afx_msg void OnUpdateUvLinesSelected(CCmdUI *pCmdUI);
	afx_msg void OnSizing(UINT fwSide, LPRECT pRect);
	afx_msg void OnHelpAboutArt();
	afx_msg void OnUpdateSomethingToSave(CCmdUI *pCmdUI);
	CStatic m_staticProtocolsSelected;
	afx_msg void OnBnClickedRadioVariance();
	afx_msg void OnBnClickedRadioStandardDeviation();
	afx_msg void OnBnClickedCheckPopulationEstimation();
	afx_msg void OnBnClickedRadioSumme();
	CButton m_checkFileEnd;
	afx_msg void OnBnClickedCheckFileEnd();

	void showCondition( int conditionNumber, BOOL setSpin = TRUE );
	CEdit m_editFileInPreview;
	CSpinButtonCtrl m_spinFileInPreview;
	afx_msg void OnDeltaposSpinFileInPreview(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnEnChangeEditFileInPreview();
	afx_msg void OnEnChangeEditCondNumber();

	void Serialize( CArchive& archive );
	CStatic m_staticFile;
	afx_msg void OnBnClickedButtonTest1();

	int actProtocolFilenumber;
	CArray<ProtocolFileChange *> protocolFileChanges;
	void deleteProtocolFileRow( int rowNumber );
	void deleteProtocolFileColumn( int columnNumber );
	void protocolFindReplace( CString origString, CString replaceString );
	void applyProtocolFileChanges( WTextFile *theTextFile );
	void undoLastProtocolFileChange( );
	afx_msg void OnBnClickedButtonTest2();
	afx_msg void OnProtocolfilesDeleteselectedrows();
	afx_msg void OnProtocolfilesDeleteselectedcolumns();
	afx_msg void OnProtocolfilesUndolastchange();
	afx_msg void OnUpdateProtocolfilesUndolastchange(CCmdUI *pCmdUI);
	afx_msg void OnUpdateProtocolfilesDeleteselectedrows(CCmdUI *pCmdUI);
	afx_msg void OnUpdateProtocolfilesDeleteselectedcolumns(CCmdUI *pCmdUI);
	afx_msg void OnProtocolfilesFindandreplace();
	afx_msg void OnBnClickedRadioRawValues();
};
